# frozen_string_literal: true
# typed: true

module ApiGenerator
  class Client
    extend T::Sig

    attr_reader :url
    attr_accessor :rest_method

    REST_METHODS = %i[
      put
      get
      patch
      delete
      post
    ]

    sig { params(url: ApiGenerator::Url, rest_method: Symbol).void }
    def initialize(url: ApiGenerator::Url.new(""), rest_method: :get)
      @url = T.let(url, ApiGenerator::Url)
      @rest_method = T.let(rest_method, Symbol)
    end


    sig { params(options: T::Hash[Symbol, T.any(String, Symbol, Integer, Date, OpenSSL::ASN1::Boolean)]).returns(Faraday::Response) }
    def execute(options)
      Faraday.send(rest_method) do |req|
        req.params = options
        req.url = url.url
      end
    end

    sig { params(method: Symbol, args: T.untyped, block: T.nilable(T.proc.void)).returns(T.self_type) }
    def method_missing(method, *args, &block)
      if REST_METHODS.any?(method)
        return ApiGenerator::Client.new(url: url, rest_method: method)
      end

      ApiGenerator::Client.new(url: ApiGenerator::Url.send(url), rest_method: rest_method)
    end
  end
end